#ifndef DUNGEON_H
#define DUNGEON_H

#include <OS.h>
#include <Locker.h>
#include <View.h>

#include "CreatureAddon.h"
#include "ObjectAddon.h"
#include "Graphics.h"
#include "LnList.h"
#include "LnFifo.h"

#include "Object.h"
#include "IdSet.h"
#include "Creature.h"
#include "misc.h"

class Dungeon;

class Tile
{
	public:
		Tile();
		
		uint32		bitmap;

		uint32		flags;
		IdSet		objects;
		uint32		creature;
		
		Dungeon 	* dungeon;
		
		void Draw( BView *, BPoint );
};

class Room
{
	public:
		int ent_x, ent_y;
		direction dir;
};

class Dungeon
{
	public:
		Dungeon();
		~Dungeon();
		
		void 		Lock();
		void 		Unlock();

		void 		Generate(int xs = 256, int ys = 256); // size; x,y

		bool 		Load( const char * filename );
		bool 		Save( const char * filename );
		
		Tile *		TileAt( Pos );

		bool		MoveCreature( uint32, Pos, Pos ); // id; from; to
		Creature *	FindCreature( uint32 );

		Object *	FindObject( uint32 );
		void		AddObject( Object *, Pos ); // thing, x,y
		void		AddObject( Object * ); // thing, add only to list not map
		Object *	CreateObject( const char * ); // create and add to list
		
		void		DoLOS(Pos, float, uint32, direction = ALL); // x,y, range, flags
		List<Pos> *	FindPath( Pos, Pos, int max_steps=50 ); // from, to
		
		void		ClearBits(uint32);

		void		DoAI(); // call Creature::AI() for all creatures in dungeon
		
		bool		CastRay(Pos, Pos, uint32);// from, to, flag
		bool		CastRay(FloatPos, FloatPos, uint32);// from, to, flag
		
		int			Width();
		int			Height();
		
	protected:
		friend		class Tile;
		friend		class Creature;
		friend		class PlayerView;
		friend		class IconView;
				
		void		CheckLock(); // check if locked, if not exit app with warning
		BLocker		lock;

		Graphics	graphics;
		int			mapsize_x, mapsize_y;
		Tile		* map;

		CreatureInfoSet	creature_info;
		ObjectInfoSet	object_info;
		
		CreatureSet	creatures;
		ObjectSet	objects;
		
		bool		CreateRoom( Room, FIFO<Room> &);
};

extern Dungeon * current_dungeon;

#endif
